// $Id: ClockView.cpp,v 1.1 1996/05/27 21:37:28 ethyl Exp ethyl $
// ClockView.cpp
//

#include "ClockView.h"
#include "ClockJDefs.h"
#include <stdio.h>

ClockView::ClockView( BRect r )
	: BView( r, "ClockView", B_FOLLOW_ALL, B_WILL_DRAW|B_PULSE_NEEDED )
{
	InitObject();
}

ClockView::ClockView( BMessage *data )
	: BView( data )
{
	InitObject();
}

ClockView::~ClockView()
{
}

void
ClockView::InitObject( void )
{
	InitFont();

	// Build pop-up menu.
	mFontMenu = new BPopUpMenu( "Font" );
	BMenuItem *item;
	item = new BMenuItem( "This menu doesn't work yet", NULL );
	item->SetEnabled( FALSE );
	mFontMenu->AddItem( item );
	mFontMenu->AddSeparatorItem();
	for ( int i=0; i<mNumFontFamilies; i++ ) {
		item = new BMenuItem( mFontFamilies[i], NULL );
		mFontMenu->AddItem( item );
	}

	// Check to see if the default font is installed.
	if ( (item = mFontMenu->FindItem( DEFAULT_FONT_FAMILY )) ) {
		printf( "Default font is found and is %s\n", item->Label() );
		item->SetMarked( TRUE );
		SetFontFamily( item->Label() );
	} else {
		printf( "Default font doesn't seem to be installed.\n" );
		printf( "Will use whatever is available.\n" );
	}

	SetFontFamily( DEFAULT_FONT_FAMILY );
}

status_t
ClockView::Archive( BMessage *data, bool deep ) const
{
	inherited::Archive( data, deep );
	data->AddString( "add_on_signature", CLOCK_J_SIGNATURE );
	data->AddString( "add_on_path", CLOCK_J_PATH );
	return 0;
}

ClockView*
ClockView::Instantiate( BMessage *data )
{
	if ( !validate_instantiation( data, "ClockView" ) )
		return NULL;
	return new ClockView( data );
}

void
ClockView::AttachedToWindow( void )
{
	SetViewColor( 200, 200, 200, 0 );
	Refresh();
}

void
ClockView::Draw( BRect r )
{
	MovePenTo( 0, 12 );
	DrawString( mTimeStr );
}

void
ClockView::MessageReceived( BMessage *m )
{
	printf( "ClockView::MessageReceived\n" );
	switch ( m->what ) {
	default:
		inherited::MessageReceived( m );
	}
}

void
ClockView::MouseDown( BPoint p )
{
	BMenuItem *selected;

	ConvertToScreen( &p );
	selected = mFontMenu->Go( p );

	if ( selected ) {
		selected->SetMarked( !(selected->IsMarked()) );
		SetFontFamily( selected->Label() );
	}
}

void
ClockView::Pulse( void )
{
	Refresh();
}

void
ClockView::Refresh( void )
{
	if ( Window()->Lock() ) {

		time_t epic;
		time( &epic );
		struct tm *local = localtime( &epic );
		int hour, min, sec;
		hour = local->tm_hour;
		min = local->tm_min;
		sec = local->tm_sec;

		sprintf( mTimeStr, "%s%s%s%s%s%s",
						(hour/10==0)?"  ":UTF8_NUMERAL[hour/10],
//						UTF8_NUMERAL[hour/10],
						UTF8_NUMERAL[hour%10],
						UTF8_JI,
						UTF8_NUMERAL[min/10], UTF8_NUMERAL[min%10],
						UTF8_FUN );

		Invalidate();
		Sync();
		Window()->Unlock();
	}
}

void
ClockView::GetUTF8Numeral( uint8 n, char *utf8 )
{
}

void
ClockView::InitFont( void )
{
	mNumFontFamilies = count_font_families();
//	printf( "There're %d font families.\n", mNumFontFamilies );
	mFontFamilies = new font_family[mNumFontFamilies];
	for ( int i=0; i<mNumFontFamilies; i++ ) {
		get_font_family( i, &mFontFamilies[i] );
//		printf( "%s\n", mFontFamilies[i] );
		int32 styleCount = count_font_styles( mFontFamilies[i] );
		font_style *fontStyles = new font_style[styleCount];
		for ( int j=0; j<styleCount; j++ ) {
			get_font_style( mFontFamilies[i], j, &fontStyles[j] );
//			printf( "   (%s)\n", fontStyles[j] );
		}
		delete[] fontStyles;
	}
}

void
ClockView::SetFontFamily( const char *family )
{
	// Won't let you change font family to anything other
	// then default family yet. I'm not really confortable
	// with the new BFont implementation as is obvious from
	// the code snippet below. ;-) If you get rid of this
	// if statement, the code becomes extremely crash-prone.
	if ( strcmp( family, DEFAULT_FONT_FAMILY ) == 0 ) {
		mFont.SetFamilyAndStyle( family, NULL );
		mFont.SetSize( DEFAULT_FONT_SIZE );
		SetFont( &mFont );
	}
}
